﻿using System;
using System.Data;
using System.Data.SqlServerCe;
using System.Collections.Generic;
using System.Text;
using CDP_Utils;
using log4net;

namespace CDP_Database
{
    class cDBCommandsHelper
    {
        // Create a logger for use in this class
        private static readonly log4net.ILog log = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        public static DataTable commandSelect(SqlCeConnection conn, String[] fields, String dbObject, String sWhere)
        {
            SqlCeCommand command = new SqlCeCommand();
            SqlCeDataReader reader = null;
            DataTable dtable = new DataTable();

            String sSelect = "SELECT ";

            if (fields.Length == 0)
            {
                cErrorHandling.fErrorMessage("There was an error constructing the query !");
                if (log.IsErrorEnabled) log.Error("Error constructing the query - No fields for select.");
                return null;
            }

            if (dbObject.Trim().Length == 0)
            {
                cErrorHandling.fErrorMessage("There was an error constructing the query !");
                if (log.IsErrorEnabled) log.Error("Error constructing the query - No database object specified.");
                return null;
            }

            try
            {
                for (int i = 0; i < fields.Length; i++)
                {
                    sSelect += fields[i] + ", ";
                }
                sSelect = sSelect.Remove(sSelect.Length - 2) + " FROM " + dbObject;

                if (sWhere != null && (sWhere.Trim()).Length > 0)
                {
                    sSelect += " WHERE " + sWhere;
                }

                command.Connection = conn;
                command.CommandType = CommandType.Text;
                command.CommandText = sSelect;

                conn.Open();
                reader = command.ExecuteReader();
                dtable.Load(reader);
            }
            catch (Exception ex)
            {
                cErrorHandling.fErrorMessage("There was an error selecting the data !");
                if (log.IsErrorEnabled) log.Error("Error selecting the data.", ex);
                return null;
            }
            finally
            {
                command.Dispose();

                if (conn.State == ConnectionState.Open)
                {
                    conn.Close();
                }

                //if (conn != null)
                //{
                //    conn.Dispose();
                //}
            }

            return dtable;
        }

        public static DataTable commandSelect(SqlCeConnection conn, String sSelect)
        {
            SqlCeCommand command = new SqlCeCommand();
            SqlCeDataReader reader = null;
            DataTable dtable = new DataTable();

            try
            {
                command.Connection = conn;
                command.CommandType = CommandType.Text;
                command.CommandText = sSelect;

                conn.Open();
                reader = command.ExecuteReader();
                dtable.Load(reader);
            }
            catch (Exception ex)
            {
                cErrorHandling.fErrorMessage("There was an error selecting the data !");
                if (log.IsErrorEnabled) log.Error("Error selecting the data.", ex);
                return null;
            }
            finally
            {
                command.Dispose();

                if (conn.State == ConnectionState.Open)
                {
                    conn.Close();
                }

                //if (conn != null)
                //{
                //    conn.Dispose();
                //}
            }

            return dtable;
        }

        public static bool commandInsert(SqlCeConnection conn, String[] fields, String[] values, String dbObject)
        {
            SqlCeCommand command = new SqlCeCommand();

            if (fields.Length == 0)
            {
                cErrorHandling.fErrorMessage("There was an error constructing the query !");
                if (log.IsErrorEnabled) log.Error("Error constructing the query - No fields for insert.");
                return false;
            }

            if (values.Length == 0)
            {
                cErrorHandling.fErrorMessage("There was an error constructing the query !");
                if (log.IsErrorEnabled) log.Error("Error constructing the query - No values for insert.");
                return false;
            }

            if (dbObject.Trim().Length == 0)
            {
                cErrorHandling.fErrorMessage("There was an error constructing the query !");
                if (log.IsErrorEnabled) log.Error("Error constructing the query - No database object specified.");
                return false;
            }

            String sInsert = "INSERT INTO " + dbObject + " (";

            try
            {
                for (int i = 0; i < fields.Length; i++)
                {
                    sInsert += "[" + fields[i] + "], ";
                }

                sInsert = sInsert.Remove(sInsert.Length - 2) + ") VALUES (";

                for (int i = 0; i < values.Length; i++)
                {
                    sInsert += values[i] + ", ";
                }

                sInsert = sInsert.Remove(sInsert.Length - 2) + ")";

                command.Connection = conn;
                command.CommandType = CommandType.Text;
                command.CommandText = sInsert;

                conn.Open();
                command.ExecuteNonQuery();
            }
            catch (Exception ex)
            {
                cErrorHandling.fErrorMessage("There was an error inserting the data !");
                if (log.IsErrorEnabled) log.Error("Error inserting the data.", ex);
                return false;
            }
            finally
            {
                command.Dispose();

                if (conn.State == ConnectionState.Open)
                {
                    conn.Close();
                }

                //if (conn != null)
                //{
                //    conn.Dispose();
                //}
            }

            return true;
        }

        public static bool commandUpdate(SqlCeConnection conn, String[] fields, String[] values, String dbObject, String sWhere)
        {
            SqlCeCommand command = new SqlCeCommand();

            if (fields.Length == 0)
            {
                cErrorHandling.fErrorMessage("There was an error constructing the query !");
                if (log.IsErrorEnabled) log.Error("Error constructing the query - No fields for update.");
                return false;
            }

            if (values.Length == 0)
            {
                cErrorHandling.fErrorMessage("There was an error constructing the query !");
                if (log.IsErrorEnabled) log.Error("Error constructing the query - No values for update.");
                return false;
            }

            if (dbObject.Trim().Length == 0)
            {
                cErrorHandling.fErrorMessage("There was an error constructing the query !");
                if (log.IsErrorEnabled) log.Error("Error constructing the query - No database object specified.");
                return false;
            }

            if (fields.Length != values.Length)
            {
                cErrorHandling.fErrorMessage("There was an error constructing the query !");
                if (log.IsErrorEnabled) log.Error("Error constructing the query - Number of values does not match number of fields.");
                return false;
            }

            String sUpdate = "UPDATE " + dbObject + " SET ";

            try
            {
                for (int i = 0; i < fields.Length; i++)
                {
                    sUpdate +=  "[" + fields[i] + "]" + " = " + values[i] +", ";
                }
                sUpdate = sUpdate.Remove(sUpdate.Length - 2);

                if (sWhere != null && (sWhere.Trim()).Length > 0)
                {
                    sUpdate += " WHERE " + sWhere;
                }

                command.Connection = conn;
                command.CommandType = CommandType.Text;
                command.CommandText = sUpdate;

                conn.Open();
                command.ExecuteNonQuery();
            }
            catch (Exception ex)
            {
                cErrorHandling.fErrorMessage("There was an error updating data !");
                if (log.IsErrorEnabled) log.Error("Error updating data.", ex);
                return false;
            }
            finally
            {
                command.Dispose();

                if (conn.State == ConnectionState.Open)
                {
                    conn.Close();
                }

                //if (conn != null)
                //{
                //    conn.Dispose();
                //}
            }

            return true;
        }

        public static bool commandDelete(SqlCeConnection conn, String sTable, string sIdentityColumn)
        {
            SqlCeCommand command = new SqlCeCommand();
            bool Success = false;

            try
            {
                if (sTable.Trim().Length == 0)
                {
                    cErrorHandling.fErrorMessage("There was an error deleting data from table !");
                    if (log.IsErrorEnabled) log.Error("Error deleting data from table - No table object specified.");
                    return false;
                }

                string sDelete = "DELETE FROM [" + sTable + "]";

                command.Connection = conn;
                command.CommandType = CommandType.Text;
                command.CommandText = sDelete;

                conn.Open();
                command.ExecuteNonQuery();

                if (sIdentityColumn.Trim().Length != 0)
                {
                    string sIdentity = "ALTER TABLE [" + sTable + "] ALTER COLUMN [" + sIdentityColumn + "] IDENTITY (1,1)";
                    command.CommandText = sIdentity;
                    command.ExecuteNonQuery();
                }
                Success = true;
            }
            catch (Exception ex)
            {
                cErrorHandling.fErrorMessage("There was an error deleting data from table " + sTable + " !");
                if (log.IsErrorEnabled) log.Error("Error deleting data from table " + sTable, ex);
                Success = false;
            }
            finally
            {
                command.Dispose();

                if (conn.State == ConnectionState.Open)
                {
                    conn.Close();
                }

                //if (conn != null)
                //{
                //    conn.Dispose();
                //}
            }

            return Success;
        }

    }
}
