﻿using System;
using System.IO;
using System.Data;
using System.Data.SqlServerCe;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CDP_Utils;
using log4net;


namespace CDP_Database
{
    public class cDBMaintenance
    {
        // Create a logger for use in this class
        private static readonly log4net.ILog log = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        private static bool KillConnections()
        {
            try
            {
                SqlCeConnection conn = cDBConnection.Instance.getConnection();
                if (conn == null)
                {
                    return false;
                }

                if (conn.State == ConnectionState.Open)
                {
                    conn.Close();
                }
                return true;
            }
            catch (Exception ex)
            {
                cErrorHandling.fErrorMessage("Error closing connections to database !");
                if (log.IsErrorEnabled) log.Error("Error closing connections to database", ex);
                return false;
            }
        }

        private static string GetDatabaseName()
        {
            try
            {
                SqlCeConnection conn = cDBConnection.Instance.getConnection();
                if (conn == null)
                {
                    return "";
                }

                return conn.Database;
            }
            catch (Exception ex)
            {
                cErrorHandling.fErrorMessage("Error getting database name !");
                if (log.IsErrorEnabled) log.Error("Error getting database name", ex);
                return "";
            }
        }

        private static string GetConnectionString()
        {
            try
            {
                SqlCeConnection conn = cDBConnection.Instance.getConnection();
                if (conn == null)
                {
                    return "";
                }

                return conn.ConnectionString;
            }
            catch (Exception ex)
            {
                cErrorHandling.fErrorMessage("Error getting connection string !");
                if (log.IsErrorEnabled) log.Error("Error getting connection string", ex);
                return "";
            }
        }

        public static bool BackupDatabase()
        {
            try
            {
                string sSourceFile = GetDatabaseName();
                if (sSourceFile == "")
                {
                    return false;
                }

                string sBackupFolder = cApplicationConfiguration.getAppSettings("dbBackupFolder");
                if (sBackupFolder == "")
                {
                    return false;
                }

                if (!Directory.Exists(sBackupFolder))
                {
                    Directory.CreateDirectory(sBackupFolder);
                }

                if (KillConnections())
                {
                    string sDestinationFile = sBackupFolder + "CDP-DatabaseBackup" + DateTime.Now.ToBinary().ToString() + ".sdf";
                    File.Copy(sSourceFile, sDestinationFile);
                    return true;
                }
                else
                {
                    return false;
                }

            }
            catch (Exception ex)
            {
                if (log.IsErrorEnabled) log.Error("Error backing up database", ex);
                return false;
            }
        }

        public static FileInfo[] GetBackups()
        {
            try
            {
                DirectoryInfo di = new DirectoryInfo(cApplicationConfiguration.getAppSettings("dbBackupFolder"));
                return di.GetFiles("*.sdf");
            }
            catch (Exception ex)
            {
                cErrorHandling.fErrorMessage("Error getting database backups !");
                if (log.IsErrorEnabled) log.Error("Error getting database backups", ex);
                return null;
            }
        }

        public static bool RestoreDatabase(string sSourceFile)
        {
            try
            {
                string sDestinationFile = GetDatabaseName();
                if (sDestinationFile == "")
                {
                    return false;
                }

                if (KillConnections())
                {
                    File.Copy(sSourceFile, sDestinationFile,true);
                    return true;
                }
                else
                {
                    return false;
                }

            }
            catch (Exception ex)
            {
                if (log.IsErrorEnabled) log.Error("Error restoring database", ex);
                return false;
            }
        }

        public static bool CompactDatabase()
        {
            try
            {
                string sConnectionString = GetConnectionString();
                if (sConnectionString == "")
                {
                    return false;
                }

                SqlCeEngine engine = new SqlCeEngine(sConnectionString);
                engine.Compact(null);

                return true;
            }
            catch (Exception ex)
            {
                if (log.IsErrorEnabled) log.Error("Error compacting database", ex);
                return false;
            }
        }

        public static bool EmptyDatabase()
        {
            try
            {
                if (!cDBOperations.TruncateTable("Travels_Flights", "TravelToFlightsId"))
                {
                    return false;
                }

                if (!cDBOperations.TruncateTable("Flights", "FlighId"))
                {
                    return false;
                }

                if (!cDBOperations.TruncateTable("Airlines", "AirlineId"))
                {
                    return false;
                }

                if (!cDBOperations.TruncateTable("Airports", ""))
                {
                    return false;
                }

                if (!cDBOperations.TruncateTable("TravelClasses", "TravelClassId"))
                {
                    return false;
                }

                if (!cDBOperations.TruncateTable("Passports", "PassportId"))
                {
                    return false;
                }

                if (!cDBOperations.TruncateTable("Persons", "PersonId"))
                {
                    return false;
                }

                if (!cDBOperations.TruncateTable("CivilStatus", "CivilStatusId"))
                {
                    return false;
                }

                if (!cDBOperations.TruncateTable("Gender", "GenderId"))
                {
                    return false;
                }

                if (!cDBOperations.TruncateTable("Addresses", "AddressId"))
                {
                    return false;
                }

                if (!cDBOperations.TruncateTable("PostalCodes", "PostalCodeId"))
                {
                    return false;
                }

                if (!cDBOperations.TruncateTable("Cities", "CityId"))
                {
                    return false;
                }

                if (!cDBOperations.TruncateTable("Countries", "CountryId"))
                {
                    return false;
                }

                if (!cDBOperations.TruncateTable("AdministrativeRegions", "AdministrativeRegionId"))
                {
                    return false;
                }
            }
            catch (Exception ex)
            {
                if (log.IsErrorEnabled) log.Error("Error clearing database", ex);
                return false;
            }
            return true;
        }
    }
}
