﻿using System;
using System.Data;
using System.Data.SqlServerCe;
using System.Collections.Generic;
using System.Text;
using CDP_Utils;
using log4net;

namespace CDP_Database
{
    public class cDBOperations
    {
        // Create a logger for use in this class
        private static readonly log4net.ILog log = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        public static bool ValidateUser(string UserName, out string UserId)
        {
            UserId = "";

            if (log.IsInfoEnabled) log.Info("Validate user " + UserName);
            try
            {
                SqlCeConnection conn = cDBConnection.Instance.getConnection();
                if (conn == null)
                {
                    return false;
                }

                string[] fields = { "UserId" };
                string dbObject = "Users";
                string sWhere = "[UserName] = '" + UserName + "'";

                DataTable dtUsers = cDBCommandsHelper.commandSelect(conn, fields, dbObject, sWhere);

                if (dtUsers == null)
                {
                    return false;
                }

                if (dtUsers.Rows.Count == 0)
                {
                    return false;
                }

                UserId = dtUsers.Rows[0]["UserId"].ToString();
                
                //throw new Exception("Test Log4Net");
            }
            catch (Exception ex)
            {
                cErrorHandling.fErrorMessage("There was an error validating the user !");
                if (log.IsErrorEnabled) log.Error("Error validating user", ex);
                return false;
            }

            return true;
        }

        public static bool ValidatePassword(string UserId, string UserPwd)
        {
            if (log.IsInfoEnabled) log.Info("Validate password");
            try
            {
                SqlCeConnection conn = cDBConnection.Instance.getConnection();
                if (conn == null)
                {
                    return false;
                }

                string[] fields = { "UserName" };
                string dbObject = "Users";
                string sWhere = "[UserPwd] = '" + UserPwd + "' and [UserId] = " + UserId;

                DataTable dtUsers = cDBCommandsHelper.commandSelect(conn, fields, dbObject, sWhere);

                if (dtUsers == null)
                {
                    return false;
                }

                if (dtUsers.Rows.Count == 0)
                {
                    return false;
                }
            }
            catch (Exception ex)
            {
                cErrorHandling.fErrorMessage("There was an error validating the password !");
                if (log.IsErrorEnabled) log.Error("Error validating password", ex);
                return false;
            }

            return true;
        }

        public static DataTable GetUserData(string UserId)
        {
            if (log.IsInfoEnabled) log.Info("Get user data");
            try
            {
                SqlCeConnection conn = cDBConnection.Instance.getConnection();
                if (conn == null)
                {
                    return null;
                }

                string[] fields = { "UserId", "UserName", "UserPwd", "PwdExpiryDate", "PwdChangeAtLogin", "IsAppAdmin", "BeginDate", "EndDate" };
                string dbObject = "Users";
                string sWhere = "[UserId] = " + UserId;

                DataTable dtUsers = cDBCommandsHelper.commandSelect(conn, fields, dbObject, sWhere);
                return dtUsers;
            }
            catch (Exception ex)
            {
                cErrorHandling.fErrorMessage("There was an error getting user data !");
                if (log.IsErrorEnabled) log.Error("Error getting user data", ex);
                return null;
            }
        }

        public static bool ChangeUserPassword(string UserId, string UserPwd)
        {
            bool Success = false;

            if (log.IsInfoEnabled) log.Info("Change user password");

            try
            {
                SqlCeConnection conn = cDBConnection.Instance.getConnection();
                if (conn == null)
                {
                    return false;
                }

                string[] fields = { "UserPwd", "PwdChangeAtLogin" };
                string[] values = { UserPwd, "0" };
                string dbObject = "Users";
                string sWhere = "[UserId] = " + UserId;

                Success = cDBCommandsHelper.commandUpdate(conn, fields, values, dbObject, sWhere);
            }
            catch (Exception ex)
            {
                cErrorHandling.fErrorMessage("There was an error changing user password !");
                if (log.IsErrorEnabled) log.Error("Error changing user password", ex);
                return false;
            }

            return Success;
        }

        public static bool AddUser(string UserName, string UserPwd, string PwdExpiryDate,string PwdChangeAtLogin,string IsAppAdmin, string BeginDate, string EndDate)
        {
            bool Success = false;

            if (log.IsInfoEnabled) log.Info("Add User");

            try
            {
                SqlCeConnection conn = cDBConnection.Instance.getConnection();
                if (conn == null)
                {
                    return false;
                }

                string[] fields = { "UserName", "UserPwd", "PwdExpiryDate", "PwdChangeAtLogin", "IsAppAdmin", "BeginDate", "EndDate" };
                string[] values = { UserName, UserPwd, PwdExpiryDate, PwdChangeAtLogin, IsAppAdmin, BeginDate, EndDate };
                string dbObject = "Users";

                Success = cDBCommandsHelper.commandInsert(conn, fields, values, dbObject);
            }
            catch (Exception ex)
            {
                cErrorHandling.fErrorMessage("There was an error adding user !");
                if (log.IsErrorEnabled) log.Error("Error adding user", ex);
                return false;
            }

            return Success;
        }

        public static bool UpdateUser(string UserId, string UserName, string UserPwd, string PwdExpiryDate, string PwdChangeAtLogin, string IsAppAdmin, string BeginDate, string EndDate)
        {
            bool Success = false;

            if (log.IsInfoEnabled) log.Info("Update User");

            try
            {
                SqlCeConnection conn = cDBConnection.Instance.getConnection();
                if (conn == null)
                {
                    return false;
                }

                string[] fields = { "UserName", "UserPwd", "PwdExpiryDate", "PwdChangeAtLogin", "IsAppAdmin", "BeginDate", "EndDate" };
                string[] values = { UserName, UserPwd, PwdExpiryDate, PwdChangeAtLogin, IsAppAdmin, BeginDate, EndDate };
                string dbObject = "Users";
                string sWhere = "[UserId] = " + UserId;

                Success = cDBCommandsHelper.commandUpdate(conn, fields, values, dbObject, sWhere);
            }
            catch (Exception ex)
            {
                cErrorHandling.fErrorMessage("There was an error updating user !");
                if (log.IsErrorEnabled) log.Error("Error updating user", ex);
                return false;
            }

            return Success;
        }

        public static DataTable GetUsersData()
        {
            try
            {
                SqlCeConnection conn = cDBConnection.Instance.getConnection();
                if (conn == null)
                {
                    return null;
                }

                string[] fields = { "UserId", "UserName", "UserPwd", "PwdExpiryDate", "PwdChangeAtLogin", "IsAppAdmin", "BeginDate", "EndDate" };
                string dbObject = "Users";
                string sWhere = "[UserName] <> 'CSAdmin'";

                DataTable dtUsers = cDBCommandsHelper.commandSelect(conn, fields, dbObject, sWhere);
                return dtUsers;
            }
            catch (Exception ex)
            {
                cErrorHandling.fErrorMessage("There was an error getting users data !");
                if (log.IsErrorEnabled) log.Error("Error getting users data", ex);
                return null;
            }
        }

        public static DataTable GetFlightInfo(string FlightCode)
        {
            try
            {
                SqlCeConnection conn = cDBConnection.Instance.getConnection();
                if (conn == null)
                {
                    return null;
                }

                StringBuilder sb = new StringBuilder();

                sb.Append("SELECT DISTINCT ");
                sb.Append("SUBSTRING(Airlines.AirlineCode + ' ' + Flights.FlightNo + REPLICATE(' ', 9 - LEN(Airlines.AirlineCode) - LEN(Flights.FlightNo) - 1), 1, 9) AS AirLineFlightNo, ");
                sb.Append("Addresses.AddressId, CASE WHEN Addresses.UnitType = 'Apartment' THEN CAST(Addresses.UnitNumber AS nvarchar(5)) ");
                sb.Append("+ ' ' + CAST(Addresses.StreetNumber AS nvarchar(5)) + ' ' + CASE WHEN COALESCE (Addresses.StreetPredirection, '') ");
                sb.Append("= '' THEN '' ELSE Addresses.StreetPredirection + ' ' END + Addresses.StreetName + CASE WHEN COALESCE (Addresses.StreetType, '') ");
                sb.Append("= '' THEN '' ELSE ' ' + Addresses.StreetType END + CASE WHEN COALESCE (Addresses.StreetPostDirection, '') ");
                sb.Append("= '' THEN '' ELSE ' ' + Addresses.StreetPostDirection END ELSE CAST(Addresses.StreetNumber AS nvarchar(5)) ");
                sb.Append("+ ' ' + CASE WHEN COALESCE (Addresses.StreetPredirection, '') "); 
                sb.Append("= '' THEN '' ELSE Addresses.StreetPredirection + ' ' END + Addresses.StreetName + CASE WHEN COALESCE (Addresses.StreetType, '') "); 
                sb.Append("= '' THEN '' ELSE ' ' + Addresses.StreetType END + CASE WHEN COALESCE (Addresses.StreetPostDirection, '') ");
                sb.Append("= '' THEN '' ELSE ' ' + Addresses.StreetPostDirection END END AS Address, SUBSTRING(Cities.City + REPLICATE(' ', 10 - LEN(Cities.City)), 1, 10) AS City, ");
                sb.Append("SUBSTRING(Countries.CountryName + REPLICATE(' ', 9 - LEN(Countries.CountryName)), 1, 9) AS CountryName, ");
                sb.Append("SUBSTRING(AdministrativeRegions.AdministrativeRegionCode + REPLICATE(' ', 4 - LEN(AdministrativeRegions.AdministrativeRegionCode)), 1, 4) ");
                sb.Append("AS AdministrativeRegionCode, SUBSTRING(PostalCodes.PostalCode + REPLICATE(' ', 7 - LEN(PostalCodes.PostalCode)), 1, 7) AS PostalCode, CONVERT(nchar(8), ");
                sb.Append("GETDATE(), 12) AS Date,1 as Rank ");
                sb.Append("FROM Flights INNER JOIN ");
                sb.Append("Airlines ON Flights.AirlineId = Airlines.AirlineId INNER JOIN ");
                sb.Append("Persons ON Flights.PersonId = Persons.PersonId INNER JOIN ");
                sb.Append("Addresses ON Persons.AddressId = Addresses.AddressId INNER JOIN ");
                sb.Append("Cities ON Addresses.CityId = Cities.CityId INNER JOIN ");
                sb.Append("Countries ON Cities.CountryId = Countries.CountryId INNER JOIN ");
                sb.Append("AdministrativeRegions ON Cities.AdministrativeRegionId = AdministrativeRegions.AdministrativeRegionId INNER JOIN ");
                sb.Append("PostalCodes ON Cities.CityId = PostalCodes.CityId ");
                sb.Append("WHERE (Flights.FlighCode = '" + FlightCode + "') ");

                DataTable dtFlightInfo = cDBCommandsHelper.commandSelect(conn, sb.ToString());
                //dtFlightInfo.Columns["Rank"].ReadOnly = false;
                //dtFlightInfo.Columns["AddressId"].ReadOnly = false;
                return dtFlightInfo;
            }
            catch (Exception ex)
            {
                cErrorHandling.fErrorMessage("There was an error getting flight info !");
                if (log.IsErrorEnabled) log.Error("Error getting flight info", ex);
                return null;
            }
        }

        public static DataTable GetPersons(string AddressId)
        {
            try
            {
                SqlCeConnection conn = cDBConnection.Instance.getConnection();
                if (conn == null)
                {
                    return null;
                }

                StringBuilder sb = new StringBuilder();

                sb.Append("SELECT PersonId, SUBSTRING(LastName + ' ' + FirstName + ' ' + SUBSTRING((case when MiddleName is null then ' ' else MiddleName end), 1, 1) + REPLICATE(' ', 26 - LEN(LastName) - LEN(FirstName) - 3), 1, 26) AS Name, ");
                sb.Append("CONVERT(nchar(6), DateofBirth, 12) AS DateofBirth, SUBSTRING(Citizenship + REPLICATE(' ', 12 - LEN(Citizenship)), 1, 12) AS Citizenship ");
                sb.Append("FROM Persons ");
                sb.Append("WHERE (AddressId = '" + AddressId + "') ");
                sb.Append("ORDER BY PersonId ");

                DataTable dtPersons = cDBCommandsHelper.commandSelect(conn, sb.ToString());
                dtPersons.Columns["PersonId"].AutoIncrement = false;
                return dtPersons;
            }
            catch (Exception ex)
            {
                cErrorHandling.fErrorMessage("There was an error getting persons !");
                if (log.IsErrorEnabled) log.Error("Error getting persons", ex);
                return null;
            }
        }

        public static DataTable GetUSFlightInfo(string FlightCode)
        {
            try
            {
                SqlCeConnection conn = cDBConnection.Instance.getConnection();
                if (conn == null)
                {
                    return null;
                }

                StringBuilder sb = new StringBuilder();

                sb.Append("SELECT Persons.FirstName, case when Persons.MiddleName is null then ' ' else Persons.MiddleName end as MiddleName, Persons.LastName, CONVERT(nchar(6), Persons.DateofBirth, 12) AS DateofBirth, ");
                sb.Append("CASE WHEN Addresses.UnitType = 'Apartment' THEN CAST(Addresses.UnitNumber AS nvarchar(5)) + ' ' + CAST(Addresses.StreetNumber AS nvarchar(5)) ");
                sb.Append("+ ' ' + CASE WHEN COALESCE (Addresses.StreetPredirection, '') ");
                sb.Append("= '' THEN '' ELSE Addresses.StreetPredirection + ' ' END + Addresses.StreetName + CASE WHEN COALESCE (Addresses.StreetType, '') ");
                sb.Append("= '' THEN '' ELSE ' ' + Addresses.StreetType END + CASE WHEN COALESCE (Addresses.StreetPostDirection, '') ");
                sb.Append("= '' THEN '' ELSE ' ' + Addresses.StreetPostDirection END ELSE CAST(Addresses.StreetNumber AS nvarchar(5)) ");
                sb.Append("+ ' ' + CASE WHEN COALESCE (Addresses.StreetPredirection, '') ");
                sb.Append("= '' THEN '' ELSE Addresses.StreetPredirection + ' ' END + Addresses.StreetName + CASE WHEN COALESCE (Addresses.StreetType, '') ");
                sb.Append("= '' THEN '' ELSE ' ' + Addresses.StreetType END + CASE WHEN COALESCE (Addresses.StreetPostDirection, '') ");
                sb.Append("= '' THEN '' ELSE ' ' + Addresses.StreetPostDirection END END AS Address, Cities.City, AdministrativeRegions.AdministrativeRegion AS State, ");
                sb.Append("Passports.PassportIssueAuthority, Passports.PassportNo, Countries.CountryName, SUBSTRING(Airlines.AirlineCode + ' ' + Flights.FlightNo + REPLICATE(' ', ");
                sb.Append("9 - LEN(Airlines.AirlineCode) - LEN(Flights.FlightNo) - 1), 1, 9) AS AirLineFlightNo, GETDATE() AS Date ");
                sb.Append("FROM Addresses INNER JOIN ");
                sb.Append("Persons ON Addresses.AddressId = Persons.AddressId INNER JOIN ");
                sb.Append("Flights ON Persons.PersonId = Flights.PersonId INNER JOIN ");
                sb.Append("Passports ON Persons.PersonId = Passports.PersonId INNER JOIN ");
                sb.Append("Cities ON Addresses.CityId = Cities.CityId INNER JOIN ");
                sb.Append("AdministrativeRegions ON Cities.AdministrativeRegionId = AdministrativeRegions.AdministrativeRegionId INNER JOIN ");
                sb.Append("Countries ON Passports.CountryId = Countries.CountryId INNER JOIN ");
                sb.Append("Airlines ON Flights.AirlineId = Airlines.AirlineId ");
                sb.Append("WHERE (Flights.FlighCode = '" + FlightCode + "') ");

                DataTable dtUSFlightInfo = cDBCommandsHelper.commandSelect(conn, sb.ToString());
                return dtUSFlightInfo;
            }
            catch (Exception ex)
            {
                cErrorHandling.fErrorMessage("There was an error getting flight info !");
                if (log.IsErrorEnabled) log.Error("Error getting flight info", ex);
                return null;
            }
        }

        public static DataTable GetCountry()
        {
            try
            {
                SqlCeConnection conn = cDBConnection.Instance.getConnection();
                if (conn == null)
                {
                    return null;
                }

                string[] fields = { "CountryId", "CountryName" };
                string dbObject = "Countries";

                DataTable dtCountry = cDBCommandsHelper.commandSelect(conn, fields, dbObject,"");
                return dtCountry;
            }
            catch (Exception ex)
            {
                cErrorHandling.fErrorMessage("There was an error getting country !");
                if (log.IsErrorEnabled) log.Error("Error getting country", ex);
                return null;
            }
        }

        public static DataTable GetAirports(string CountryId)
        {
            try
            {
                SqlCeConnection conn = cDBConnection.Instance.getConnection();
                if (conn == null)
                {
                    return null;
                }

                string[] fields = { "AirportCode", "AirportName" };
                string dbObject = "Airports";
                string sWhere = "[CountryId] = " + CountryId;

                DataTable dtAirports = cDBCommandsHelper.commandSelect(conn, fields, dbObject, sWhere);
                return dtAirports;
            }
            catch (Exception ex)
            {
                cErrorHandling.fErrorMessage("There was an error getting airports !");
                if (log.IsErrorEnabled) log.Error("Error getting airports", ex);
                return null;
            }
        }

        public static DataTable GetFlights(string AirportCode)
        {
            try
            {
                SqlCeConnection conn = cDBConnection.Instance.getConnection();
                if (conn == null)
                {
                    return null;
                }

                StringBuilder sb = new StringBuilder();

                sb.Append("SELECT DISTINCT FlighCode, FlightNo ");
                sb.Append("FROM Flights ");
                sb.Append("WHERE (ltrim(rtrim(AirportCodeArrival)) = '" + AirportCode.Trim() + "') ");

                DataTable dtFlights = cDBCommandsHelper.commandSelect(conn, sb.ToString());
                return dtFlights;
            }
            catch (Exception ex)
            {
                cErrorHandling.fErrorMessage("There was an error getting flights !");
                if (log.IsErrorEnabled) log.Error("Error getting flights", ex);
                return null;
            }
        }

        public static DataTable GetSingleFlightInfo(string FlightCode)
        {
            try
            {
                SqlCeConnection conn = cDBConnection.Instance.getConnection();
                if (conn == null)
                {
                    return null;
                }

                StringBuilder sb = new StringBuilder();

                sb.Append("SELECT ");
                sb.Append("SUBSTRING(Persons.LastName + ' ' + Persons.FirstName + ' ' + SUBSTRING((case when Persons.MiddleName is null then ' ' else Persons.MiddleName end), 1, 1) + REPLICATE(' ', 26 - LEN(Persons.LastName) - LEN(Persons.FirstName) - 3), 1, 26) AS Name, ");
                sb.Append("CONVERT(nchar(6), Persons.DateofBirth, 12) AS DateofBirth, SUBSTRING(Persons.Citizenship + REPLICATE(' ', 12 - LEN(Persons.Citizenship)), 1, 12) AS Citizenship, ");
                sb.Append("SUBSTRING(Airlines.AirlineCode + ' ' + Flights.FlightNo + REPLICATE(' ', 9 - LEN(Airlines.AirlineCode) - LEN(Flights.FlightNo) - 1), 1, 9) AS AirLineFlightNo, ");
                sb.Append("Addresses.AddressId, CASE WHEN Addresses.UnitType = 'Apartment' THEN CAST(Addresses.UnitNumber AS nvarchar(5)) ");
                sb.Append("+ ' ' + CAST(Addresses.StreetNumber AS nvarchar(5)) + ' ' + CASE WHEN COALESCE (Addresses.StreetPredirection, '') ");
                sb.Append("= '' THEN '' ELSE Addresses.StreetPredirection + ' ' END + Addresses.StreetName + CASE WHEN COALESCE (Addresses.StreetType, '') ");
                sb.Append("= '' THEN '' ELSE ' ' + Addresses.StreetType END + CASE WHEN COALESCE (Addresses.StreetPostDirection, '') ");
                sb.Append("= '' THEN '' ELSE ' ' + Addresses.StreetPostDirection END ELSE CAST(Addresses.StreetNumber AS nvarchar(5)) ");
                sb.Append("+ ' ' + CASE WHEN COALESCE (Addresses.StreetPredirection, '') ");
                sb.Append("= '' THEN '' ELSE Addresses.StreetPredirection + ' ' END + Addresses.StreetName + CASE WHEN COALESCE (Addresses.StreetType, '') ");
                sb.Append("= '' THEN '' ELSE ' ' + Addresses.StreetType END + CASE WHEN COALESCE (Addresses.StreetPostDirection, '') ");
                sb.Append("= '' THEN '' ELSE ' ' + Addresses.StreetPostDirection END END AS Address, SUBSTRING(Cities.City + REPLICATE(' ', 10 - LEN(Cities.City)), 1, 10) AS City, ");
                sb.Append("SUBSTRING(Countries.CountryName + REPLICATE(' ', 9 - LEN(Countries.CountryName)), 1, 9) AS CountryName, ");
                sb.Append("SUBSTRING(AdministrativeRegions.AdministrativeRegionCode + REPLICATE(' ', 4 - LEN(AdministrativeRegions.AdministrativeRegionCode)), 1, 4) ");
                sb.Append("AS AdministrativeRegionCode, SUBSTRING(PostalCodes.PostalCode + REPLICATE(' ', 7 - LEN(PostalCodes.PostalCode)), 1, 7) AS PostalCode, CONVERT(nchar(8), ");
                sb.Append("GETDATE(), 12) AS Date ");
                sb.Append("FROM Flights INNER JOIN ");
                sb.Append("Airlines ON Flights.AirlineId = Airlines.AirlineId INNER JOIN ");
                sb.Append("Persons ON Flights.PersonId = Persons.PersonId INNER JOIN ");
                sb.Append("Addresses ON Persons.AddressId = Addresses.AddressId INNER JOIN ");
                sb.Append("Cities ON Addresses.CityId = Cities.CityId INNER JOIN ");
                sb.Append("Countries ON Cities.CountryId = Countries.CountryId INNER JOIN ");
                sb.Append("AdministrativeRegions ON Cities.AdministrativeRegionId = AdministrativeRegions.AdministrativeRegionId INNER JOIN ");
                sb.Append("PostalCodes ON Cities.CityId = PostalCodes.CityId ");
                sb.Append("WHERE (Flights.FlighCode = '" + FlightCode + "') ");

                DataTable dtFlightInfo = cDBCommandsHelper.commandSelect(conn, sb.ToString());
                return dtFlightInfo;
            }
            catch (Exception ex)
            {
                cErrorHandling.fErrorMessage("There was an error getting flight info !");
                if (log.IsErrorEnabled) log.Error("Error getting flight info", ex);
                return null;
            }
        }

        public static DataTable GetAddresses(string FlightCode)
        {
            try
            {
                SqlCeConnection conn = cDBConnection.Instance.getConnection();
                if (conn == null)
                {
                    return null;
                }

                StringBuilder sb = new StringBuilder();

                sb.Append("select Persons.AddressId,ceiling(cast(count(*) as money)/4.000) as NrRec ");
                sb.Append("from Persons ");
                sb.Append("inner join Flights on Persons.PersonId = Flights.PersonId ");
                sb.Append("where (Flights.FlighCode = '" + FlightCode + "') ");
                sb.Append("group by Persons.AddressId ");

                DataTable dtAddresses = cDBCommandsHelper.commandSelect(conn, sb.ToString());
                return dtAddresses;
            }
            catch (Exception ex)
            {
                cErrorHandling.fErrorMessage("There was an error getting addresses !");
                if (log.IsErrorEnabled) log.Error("Error getting addresses", ex);
                return null;
            }
        }

        public static bool TruncateTable(string sTable, string sIdentityColumn)
        {
            bool Success = false;

            if (log.IsInfoEnabled) log.Info("Truncate table " + sTable);

            try
            {
                SqlCeConnection conn = cDBConnection.Instance.getConnection();
                if (conn == null)
                {
                    return false;
                }

                Success = cDBCommandsHelper.commandDelete(conn, sTable, sIdentityColumn);
            }
            catch (Exception ex)
            {
                cErrorHandling.fErrorMessage("There was an error deleting table " + sTable + " !");
                if (log.IsErrorEnabled) log.Error("Error deleting table " + sTable, ex);
                Success = false;
            }

            return Success;
        }

    }
}
