﻿using System;
using System.Configuration;
using System.Collections.Generic;
using System.Text;
using log4net;

namespace CDP_Utils
{
    public class cApplicationConfiguration
    {
        // Create a logger for use in this class
        private static readonly log4net.ILog log = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        public static string getConnectionString(string ConnectionName)
        {
            string ConnectionString = "";

            try
            {
                Configuration config = ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None);
                ConnectionString = config.ConnectionStrings.ConnectionStrings[ConnectionName].ConnectionString;
            }
            catch (Exception ex)
            {
                cErrorHandling.fErrorMessage("Error getting connection string !");
                if (log.IsErrorEnabled) log.Error("Error getting connection string", ex);
                return string.Empty;
            }

            if (log.IsInfoEnabled) log.Info("Get connection string");
            return ConnectionString;
        }

        public static string getAppSettings(string Key)
        {
            string KeyValue = "";

            try
            {
                Configuration config = ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None);
                KeyValue = config.AppSettings.Settings[Key].Value;
            }
            catch (Exception ex)
            {
                cErrorHandling.fErrorMessage("Error getting application settings !");
                if (log.IsErrorEnabled) log.Error("Error getting application settings", ex);
                return string.Empty;
            }

            if (log.IsInfoEnabled) log.Info("Get application settings");
            return KeyValue;
        }

        public static string getDecryptedAppSettings(string Key)
        {
            string KeyValue = "";

            try
            {
                Configuration config = ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None);
                KeyValue = cCrypto.Decrypt(config.AppSettings.Settings[Key].Value);
            }
            catch (Exception ex)
            {
                cErrorHandling.fErrorMessage("Error getting application settings !");
                if (log.IsErrorEnabled) log.Error("Error getting decrypted application settings", ex);
                return string.Empty;
            }

            if (log.IsInfoEnabled) log.Info("Get decrypted application settings");
            return KeyValue;
        }
    }
}
