﻿using System;
using System.IO;
using System.Windows.Forms;
using System.Security.Cryptography;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using log4net;

namespace CDP_Utils
{
    public class cCrypto
    {
        // Create a logger for use in this class
        private static readonly log4net.ILog log = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        protected internal static string Decrypt(string EncryptedValue)
        {
            string DecryptedValue = string.Empty;

            try
            {
                DecryptedValue = DecryptStringUsingXMLFile(EncryptedValue, GetPath());
            }
            catch (Exception ex)
            {
                cErrorHandling.fErrorMessage("Error decrypting information !");
                if (log.IsErrorEnabled) log.Error("Error decrypting information", ex);
                return string.Empty;
            }

            if (log.IsInfoEnabled) log.Info("Information decrypted");
            return DecryptedValue;
        }

        private static string DecryptStringUsingRegistryKey(string decryptValue, string privateKey)
        {
            // This is the variable that will be returned to the user
            string decryptedValue = string.Empty;

            try
            {
                // Make sure user supplied a value for the registry key
                if (string.IsNullOrEmpty(privateKey))
                    throw new ArgumentNullException("You must provide the name of the registry key for the public key");

                // Create the CspParameters object which is used to create the RSA provider
                // without it generating a new private/public key.
                // Parameter value of 1 indicates RSA provider type - 13 would indicate DSA provider
                CspParameters csp = new CspParameters(1);

                // Registry key name containing the RSA private/public key
                csp.KeyContainerName = privateKey;

                // Supply the provider name
                csp.ProviderName = "Microsoft Strong Cryptographic Provider";

                //Create new RSA object passing our key info
                RSACryptoServiceProvider rsa = new RSACryptoServiceProvider(csp);

                // Before decryption we must convert this ugly string into a byte array
                byte[] valueToDecrypt = Convert.FromBase64String(decryptValue);

                // Decrypt the passed in string value - Again the false value has to do with padding
                byte[] plainTextValue = rsa.Decrypt(valueToDecrypt, false);

                // Extract our decrypted byte array into a string value to return to our user
                decryptedValue = System.Text.Encoding.UTF8.GetString(plainTextValue);

            }
            catch (Exception ex)
            {
                cErrorHandling.fErrorMessage("Error decrypting information !");
                if (log.IsErrorEnabled) log.Error("Error decrypting information", ex);
                return string.Empty;
            }

            if (log.IsInfoEnabled) log.Info("Information decrypted using registry key");
            return decryptedValue;
        }

        public static string DecryptStringUsingXMLFile(string decryptValue, string privateKeyPath)
        {
            // This is the variable that will be returned to the user
            string decryptedValue = string.Empty;

            try
            {
                // Variable to hold contents of private key xml
                string privateKey;

                // Make sure user supplied a value for the registry key
                if (string.IsNullOrEmpty(privateKeyPath))
                    throw new ArgumentNullException("You must provide the name of the registry key for the public key");

                // Read public key from xml file
                using (StreamReader reader = new StreamReader(privateKeyPath))
                {
                    privateKey = reader.ReadToEnd();
                }

                // Create the CspParameters object which is used to create the RSA provider
                // without it generating a new private/public key.
                // Parameter value of 1 indicates RSA provider type - 13 would indicate DSA provider
                CspParameters csp = new CspParameters(1);

                // Supply the provider name
                csp.ProviderName = "Microsoft Strong Cryptographic Provider";

                //Create new RSA object passing our key info
                RSACryptoServiceProvider rsa = new RSACryptoServiceProvider(csp);
                rsa.FromXmlString(privateKey);

                // Before decryption we must convert this ugly string into a byte array
                byte[] valueToDecrypt = Convert.FromBase64String(decryptValue);

                // Decrypt the passed in string value - Again the false value has to do with padding
                byte[] plainTextValue = rsa.Decrypt(valueToDecrypt, false);

                // Extract our decrypted byte array into a string value to return to our user
                decryptedValue = System.Text.Encoding.UTF8.GetString(plainTextValue);
            }
            catch (Exception ex)
            {
                cErrorHandling.fErrorMessage("Error decrypting information !");
                if (log.IsErrorEnabled) log.Error("Error decrypting information", ex);
                return string.Empty;
            }

            if (log.IsInfoEnabled) log.Info("Information decrypted using XML file");
            return decryptedValue;
        }

        private static string GetPath()
        {
            try
            {
                byte[] keyArray;
                byte[] toDecryptArray = Convert.FromBase64String(cApplicationConfiguration.getAppSettings("dbPath"));

                MD5CryptoServiceProvider hashmd = new MD5CryptoServiceProvider();
                keyArray = hashmd.ComputeHash(UTF8Encoding.UTF8.GetBytes(Application.CompanyName + " - " + Application.ProductName));
            
                hashmd.Clear();

                TripleDESCryptoServiceProvider tDes = new TripleDESCryptoServiceProvider();
                tDes.Key = keyArray;
                tDes.Mode = CipherMode.ECB;
                tDes.Padding = PaddingMode.PKCS7;
                ICryptoTransform cTransform = tDes.CreateDecryptor();

                byte[] resultArray = cTransform.TransformFinalBlock(toDecryptArray, 0, toDecryptArray.Length);

                tDes.Clear();

                if (log.IsInfoEnabled) log.Info("Information decrypted");
                return UTF8Encoding.UTF8.GetString(resultArray, 0, resultArray.Length);
            }
            catch (Exception ex)
            {
                cErrorHandling.fErrorMessage("Error decrypting information !");
                if (log.IsErrorEnabled) log.Error("Error decrypting information", ex);
                return string.Empty;
            }
        }

    }
}
